@@ -9,26 +9,19 @@
 import os
 import pickle
 
-from functools import partial
-from hashlib import md5
-
 import attr
 
-from commoncode import ignore
 from commoncode.datautils import attribute
-from commoncode.fileutils import resource_iter
 from commoncode.fileutils import create_dir
 
 from scancode_config import licensedcode_cache_dir
 from scancode_config import scancode_cache_dir
-from scancode_config import scancode_src_dir
-from scancode_config import SCANCODE_DEV_MODE
 
 """
 An on-disk persistent cache of LicenseIndex and related data structures such as
-the licenses database. The data are  pickled and invalidated if there are any
-changes in the code or licenses text or rules. Loading and dumping the cached
-pickle is safe to use across multiple processes using lock files.
+the licenses database. The data are pickled and must be regenerated if there
+are any changes in the code or licenses text or rules. Loading and dumping the
+cached pickle is safe to use across multiple processes using lock files.
 """
 
 # This is the Pickle protocol we use, which was added in Python 3.4.
@@ -59,10 +52,10 @@ class LicenseCache:
     def load_or_build(
         licensedcode_cache_dir=licensedcode_cache_dir,
         scancode_cache_dir=scancode_cache_dir,
-        check_consistency=SCANCODE_DEV_MODE,
+        force=False,
+        index_all_languages=False,
         # used for testing only
         timeout=LICENSE_INDEX_LOCK_TIMEOUT,
-        tree_base_dir=scancode_src_dir,
         licenses_data_dir=None,
         rules_data_dir=None,
     ):
@@ -73,11 +66,11 @@ class LicenseCache:
         On the side, we load cached or build license db, SPDX symbols and other
         license-related data structures.
 
+        - If the cache exists, it is returned unless corrupted or ``force`` is True.
         - If the cache does not exist, a new index is built and cached.
-        - If `check_consistency` is True, the cache is checked for consistency and
-          rebuilt if inconsistent or stale.
-        - If `check_consistency` is False, the cache is NOT checked for consistency and
-          if the cache files exist but ARE stale, the cache WILL NOT be rebuilt
+        - If ``index_all_languages`` is True, include texts in all languages when
+          building the license index. Otherwise, only include the English license \
+          texts and rules (the default)
         """
         idx_cache_dir = os.path.join(licensedcode_cache_dir, LICENSE_INDEX_DIR)
         create_dir(idx_cache_dir)
@@ -85,14 +78,14 @@ class LicenseCache:
 
         has_cache = os.path.exists(cache_file) and os.path.getsize(cache_file)
 
-        # bypass check if no consistency check is needed
-        if has_cache and not check_consistency:
+        # bypass build if cache exists
+        if has_cache and not force:
             try:
                 return load_cache_file(cache_file)
             except Exception as e:
                 # work around some rare Windows quirks
                 import traceback
-                print('Inconsistent License cache: checking and rebuilding index.')
+                print('Inconsistent License cache: rebuilding index.')
                 print(str(e))
                 print(traceback.format_exc())
 
@@ -105,39 +98,23 @@ class LicenseCache:
         rules_data_dir = rules_data_dir or rdd
 
         lock_file = os.path.join(scancode_cache_dir, LICENSE_LOCKFILE_NAME)
-        checksum_file = os.path.join(scancode_cache_dir, LICENSE_CHECKSUM_FILE)
 
-        has_tree_checksum = os.path.exists(checksum_file)
-
-        # here, we have no cache or we want a validity check: lock, check
-        # and build or rebuild as needed
+        # here, we have no cache: lock, check and rebuild
         try:
             # acquire lock and wait until timeout to get a lock or die
             with lockfile.FileLock(lock_file).locked(timeout=timeout):
-                current_checksum = None
-                # is the current cache consistent or stale?
-                if has_cache and has_tree_checksum:
-                    # if we have a saved cached index
-                    # load saved tree_checksum and compare with current tree_checksum
-                    with open(checksum_file) as etcs:
-                        existing_checksum = etcs.read()
-
-                    current_checksum = tree_checksum(tree_base_dir=tree_base_dir)
-                    if current_checksum == existing_checksum:
-                        # The cache is consistent with the latest code and data
-                        # load and return
-                        return load_cache_file(cache_file)
-
-                # Here, the cache is not consistent with the latest code and
-                # data: It is either stale or non-existing: we need to
+                # Here, the cache is either stale or non-existing: we need to
                 # rebuild all cached data (e.g. mostly the index) and cache it
 
                 licenses_db = load_licenses(licenses_data_dir=licenses_data_dir)
+
                 index = build_index(
                     licenses_db=licenses_db,
                     licenses_data_dir=licenses_data_dir,
                     rules_data_dir=rules_data_dir,
+                    index_all_languages=index_all_languages,
                 )
+
                 spdx_symbols = build_spdx_symbols(licenses_db=licenses_db)
                 unknown_spdx_symbol = build_unknown_spdx_symbol(licenses_db=licenses_db)
                 licensing = build_licensing(licenses_db=licenses_db)
@@ -154,10 +131,6 @@ class LicenseCache:
                 with open(cache_file, 'wb') as fn:
                     pickle.dump(license_cache, fn, protocol=PICKLE_PROTOCOL)
 
-                current_checksum = tree_checksum(tree_base_dir=tree_base_dir)
-                with open(checksum_file, 'w') as ctcs:
-                    ctcs.write(current_checksum)
-
                 return license_cache
 
         except lockfile.LockTimeout:
@@ -165,24 +138,48 @@ class LicenseCache:
             raise
 
 
-def build_index(licenses_db=None, licenses_data_dir=None, rules_data_dir=None):
+def build_index(
+    licenses_db=None,
+    licenses_data_dir=None,
+    rules_data_dir=None,
+    index_all_languages=False,
+):
     """
     Return an index built from rules and licenses directories
+
+    If ``index_all_languages`` is True, include texts and rules in all languages.
+    Otherwise, only include the English license texts and rules (the default)
     """
     from licensedcode.index import LicenseIndex
     from licensedcode.models import get_rules
     from licensedcode.models import get_all_spdx_key_tokens
+    from licensedcode.models import get_license_tokens
     from licensedcode.models import licenses_data_dir as ldd
     from licensedcode.models import rules_data_dir as rdd
     from licensedcode.models import load_licenses
+    from licensedcode.legalese import common_license_words
 
     licenses_data_dir = licenses_data_dir or ldd
     rules_data_dir = rules_data_dir or rdd
 
     licenses_db = licenses_db or load_licenses(licenses_data_dir=licenses_data_dir)
-    spdx_tokens = set(get_all_spdx_key_tokens(licenses_db))
     rules = get_rules(licenses_db=licenses_db, rules_data_dir=rules_data_dir)
-    return LicenseIndex(rules, _spdx_tokens=spdx_tokens)
+
+    legalese = common_license_words
+    spdx_tokens = set(get_all_spdx_key_tokens(licenses_db))
+    license_tokens = set(get_license_tokens())
+
+    # only skip licenses to be indexed
+    if not index_all_languages:
+        rules = (r for r in rules if r.language == 'en')
+
+    return LicenseIndex(
+        rules,
+        _legalese=legalese,
+        _spdx_tokens=spdx_tokens,
+        _license_tokens=license_tokens,
+        _all_languages=index_all_languages,
+    )
 
 
 def build_licensing(licenses_db=None):
@@ -208,37 +205,87 @@ def build_spdx_symbols(licenses_db=None):
     from licensedcode.models import load_licenses
 
     licenses_db = licenses_db or load_licenses()
-    symbols_by_spdx_key = {}
 
-    for lic in licenses_db.values():
+    licenses_by_spdx_key = get_licenses_by_spdx_key(
+        licenses=licenses_db.values(),
+        include_deprecated=False,
+        lowercase_keys=True,
+        include_other_spdx_license_keys=True,
+    )
+
+    return {
+        spdx: LicenseSymbolLike(lic)
+        for spdx, lic in licenses_by_spdx_key.items()
+    }
+
+
+def get_licenses_by_spdx_key(
+    licenses=None,
+    include_deprecated=False,
+    lowercase_keys=True,
+    include_other_spdx_license_keys=False,
+):
+    """
+    Return a mapping of {SPDX license id: License} where license is a License
+    object loaded from a `licenses` list of License or the standard
+    license db if not provided.
+
+    Optionally include deprecated if ``include_deprecated`` is True.
+
+
+    Optionally make the keys lowercase if ``lowercase_keys`` is True.
+
+    Optionally include the license "other_spdx_license_keys" if present and
+    ``include_other_spdx_license_keys`` is True.
+    """
+    from licensedcode.models import load_licenses
+
+    if not licenses:
+        licenses = load_licenses().values()
+
+    licenses_by_spdx_key = {}
+
+    for lic in licenses:
         if not (lic.spdx_license_key or lic.other_spdx_license_keys):
             continue
 
-        symbol = LicenseSymbolLike(lic)
         if lic.spdx_license_key:
-            slk = lic.spdx_license_key.lower()
-            existing = symbols_by_spdx_key.get(slk)
+            slk = lic.spdx_license_key
+            if lowercase_keys:
+                slk = slk.lower()
+            existing = licenses_by_spdx_key.get(slk)
+            if existing and not lic.is_deprecated:
+                # temp hack for wharty ICU key!!
+                if slk not in ('icu', 'ICU',):
+                    raise ValueError(
+                        f'Duplicated SPDX license key: {slk!r} defined in '
+                        f'{lic.key!r} and {existing!r}'
+                    )
 
-            if existing:
-                raise ValueError(
-                    'Duplicated SPDX license key: %(slk)r defined in '
-                    '%(lic)r and %(existing)r' % locals())
+            if (
+                not lic.is_deprecated
+                or (lic.is_deprecated and include_deprecated)
+            ):
+                licenses_by_spdx_key[slk] = lic
 
-            symbols_by_spdx_key[slk] = symbol
+        if include_other_spdx_license_keys:
+            for other_spdx in lic.other_spdx_license_keys:
+                if not other_spdx or not other_spdx.strip():
+                    continue
+                slk = other_spdx
+                if lowercase_keys:
+                    slk = slk.lower()
 
-        for other_spdx in lic.other_spdx_license_keys:
-            if not (other_spdx and other_spdx.strip()):
-                continue
-            slk = other_spdx.lower()
-            existing = symbols_by_spdx_key.get(slk)
+                existing = licenses_by_spdx_key.get(slk)
+                if existing:
+                    raise ValueError(
+                        f'Duplicated "other" SPDX license key: {slk!r} defined '
+                        f'in {lic.key!r} and {existing!r}'
+                    )
 
-            if existing:
-                raise ValueError(
-                    'Duplicated "other" SPDX license key: %(slk)r defined '
-                    'in %(lic)r and %(existing)r' % locals())
-            symbols_by_spdx_key[slk] = symbol
+                licenses_by_spdx_key[slk] = lic
 
-    return symbols_by_spdx_key
+    return licenses_by_spdx_key
 
 
 def build_unknown_spdx_symbol(licenses_db=None):
@@ -252,28 +299,32 @@ def build_unknown_spdx_symbol(licenses_db=None):
     return LicenseSymbolLike(licenses_db['unknown-spdx'])
 
 
-def get_cache(check_consistency=SCANCODE_DEV_MODE):
+def get_cache(force=False, index_all_languages=False):
     """
-    Optionally return and either load or build and cache a LicenseCache.
+    Return a LicenseCache either rebuilt, cached or loaded from disk.
+
+    If ``index_all_languages`` is True, include texts in all languages when
+    building the license index. Otherwise, only include the English license \
+    texts and rules (the default)
     """
-    populate_cache(check_consistency=check_consistency)
+    populate_cache(force=force, index_all_languages=index_all_languages)
     global _LICENSE_CACHE
     return _LICENSE_CACHE
 
 
-def populate_cache(check_consistency=SCANCODE_DEV_MODE):
+def populate_cache(force=False, index_all_languages=False):
     """
     Load or build and cache a LicenseCache. Return None.
     """
     global _LICENSE_CACHE
-    if not _LICENSE_CACHE:
+    if force or not _LICENSE_CACHE:
         _LICENSE_CACHE = LicenseCache.load_or_build(
             licensedcode_cache_dir=licensedcode_cache_dir,
             scancode_cache_dir=scancode_cache_dir,
-            check_consistency=check_consistency,
+            force=force,
+            index_all_languages=index_all_languages,
             # used for testing only
             timeout=LICENSE_INDEX_LOCK_TIMEOUT,
-            tree_base_dir=scancode_src_dir,
         )
 
 
@@ -295,71 +346,58 @@ def load_cache_file(cache_file):
             raise Exception(msg) from e
 
 
-_ignored_from_hash = partial(
-    ignore.is_ignored,
-    ignores={
-        '*.pyc': 'pyc files',
-        '*~': 'temp gedit files',
-        '*.swp': 'vi swap files',
-    },
-    unignores={}
-)
-
-licensedcode_dir = os.path.join(scancode_src_dir, 'licensedcode')
-
-
-def tree_checksum(tree_base_dir=licensedcode_dir, _ignored=_ignored_from_hash):
-    """
-    Return a checksum computed from a file tree using the file paths, size and
-    last modified time stamps. The purpose is to detect is there has been any
-    modification to source code or data files and use this as a proxy to verify
-    the cache consistency. This includes the actual cached index file.
-
-    NOTE: this is not 100% fool proof but good enough in practice.
-    """
-    resources = resource_iter(tree_base_dir, ignored=_ignored, with_dirs=False)
-    hashable = (pth + str(os.path.getmtime(pth)) + str(os.path.getsize(pth)) for pth in resources)
-    hashable = ''.join(sorted(hashable))
-    hashable = hashable.encode('utf-8')
-    return md5(hashable).hexdigest()
-
-
-def get_index(check_consistency=SCANCODE_DEV_MODE):
+def get_index(force=False, index_all_languages=False):
     """
     Return and eventually build and cache a LicenseIndex.
     """
-    return get_cache(check_consistency=check_consistency).index
+    return get_cache(force=force, index_all_languages=index_all_languages).index
 
 
 get_cached_index = get_index
 
 
-def get_licenses_db(check_consistency=SCANCODE_DEV_MODE):
+def get_licenses_db():
     """
     Return a mapping of license key -> license object.
     """
-    return get_cache(check_consistency=check_consistency).db
+    return get_cache().db
 
 
-def get_licensing(check_consistency=SCANCODE_DEV_MODE):
+def get_licensing():
     """
     Return a license_expression.Licensing objet built from the all the licenses.
     """
-    return get_cache(check_consistency=check_consistency).licensing
+    return get_cache().licensing
 
 
-def get_unknown_spdx_symbol(check_consistency=SCANCODE_DEV_MODE):
+def get_unknown_spdx_symbol():
     """
     Return the unknown SPDX license symbol.
     """
-    return get_cache(check_consistency=check_consistency).unknown_spdx_symbol
+    return get_cache().unknown_spdx_symbol
 
 
-def get_spdx_symbols(licenses_db=None, check_consistency=SCANCODE_DEV_MODE):
+def get_spdx_symbols(licenses_db=None):
     """
     Return a mapping of {lowercased SPDX license key: LicenseSymbolLike} where
     LicenseSymbolLike wraps a License object
     """
     if licenses_db:
         return build_spdx_symbols(licenses_db)
-    return get_cache(check_consistency=check_consistency).spdx_symbols
+    return get_cache().spdx_symbols
+
+
+def build_spdx_license_expression(license_expression, licensing=None):
+    """
+    Return an SPDX license expression from a ScanCode ``license_expression``
+    string.
+
+    For example::
+    >>> exp = "mit OR gpl-2.0 with generic-exception"
+    >>> spdx = "MIT OR GPL-2.0-only WITH LicenseRef-scancode-generic-exception"
+    >>> assert build_spdx_license_expression(exp) == spdx
+    """
+    if not licensing:
+        licensing = get_licensing()
+    parsed = licensing.parse(license_expression)
+    return parsed.render(template='{symbol.wrapped.spdx_license_key}')
