@@ -19,7 +19,7 @@ from licensedcode.spans import Span
 Matching strategy for exact matching using Aho-Corasick automatons.
 """
 
-# Set to False to enable debug tracing
+# Set to True to enable debug tracing
 TRACE = False
 TRACE_FRAG = False
 TRACE_DEEP = False
@@ -93,7 +93,13 @@ def exact_match(idx, query_run, automaton, matcher=MATCH_AHO_EXACT, **kwargs):
     qbegin = query_run.start
 
     matched_positions = get_matched_positions(query_run.tokens, qbegin, automaton)
+    if TRACE:
+        matched_positions = list(matched_positions)
+        logger_debug(' ##exact_AHO: matched_positions', matched_positions)
     matched_spans = get_matched_spans(matched_positions, query_run.matchables)
+    if TRACE:
+        matched_spans = list(matched_spans)
+        logger_debug(' ##exact_AHO: matched_spans', matched_spans)
 
     len_legalese = idx.len_legalese
     rules_by_rid = idx.rules_by_rid
@@ -126,7 +132,7 @@ def get_matched_spans(positions, matchables):
         qspan = Span(list(range(match_qstart, match_qend)))
         # TODO: this should be optimized?
         # e.g. with not qspan.set.issubset(matchables):
-        if any(p not in matchables for p in qspan):  
+        if any(p not in matchables for p in qspan):
             if TRACE: logger_debug(
                 '   #exact_AHO:get_matched_spans not matchable match:',
                 'any(p not in matchables for p in qspan)',
@@ -155,9 +161,9 @@ def get_matched_positions(tokens, qbegin, automaton):
 
 def get_matches(tokens, qbegin, automaton):
     """
-    Yield tuples of automaton matches positions as (match end, match value) from
-    matching `tokens` sequence of token ids starting at the `qbegin` absolute
-    query start position position using the `automaton`.
+    Yield tuples of automaton matches as (match end, match value) from matching
+    the ``tokens`` sequence of token ids starting at the `qbegin` absolute query
+    start position position using the `automaton`.
     """
     # iterate over matched strings: the matched value is (rule id, index start
     # pos, index end pos)
@@ -186,9 +192,8 @@ def match_fragments(idx, query_run):
         for m in matches:
             print(m)
 
-    # Discard fragments that have any already matched positions in previous matches
-    from licensedcode.match import filter_already_matched_matches
-    matches, _discarded = filter_already_matched_matches(matches, query_run.query)
+    # Discard fragments that have any already matched positions in any previous matches
+    matches, _discarded = filter_already_matched_overlapping_matches(matches, query_run.query)
 
     # Merge matches with a zero max distance, e.g. contiguous or overlapping
     # with matches to the same rule
@@ -239,6 +244,30 @@ def match_fragments(idx, query_run):
     return frag_matches
 
 
+def filter_already_matched_overlapping_matches(matches, query):
+    """
+    Return a filtered list of kept LicenseMatch matches and a list of
+    discardable matches given a `matches` list of LicenseMatch by removing
+    matches that have at least one position that is already matched in the Query
+    `query`.
+    """
+    kept = []
+    discarded = []
+
+    matched_pos = query.matched
+    for match in matches:
+        # FIXME: do not use internals!!!
+        match_qspan_set = match.qspan._set
+        if match_qspan_set & matched_pos:
+            # discard any match that has any position already matched
+            if TRACE_FRAG: logger_debug('    ==> DISCARDING ALREADY MATCHED:', match)
+            discarded.append(match)
+        else:
+            kept.append(match)
+
+    return kept, discarded
+
+
 def add_start(automaton, tids, rule_identifier, rule_length):
     """
     Add the `tids` sequence of starting token ids for the `rule_identifier` Rule
