@@ -7,7 +7,12 @@
 # See https://aboutcode.org for more information about nexB OSS projects.
 #
 
-from license_expression import Licensing
+try:
+    from license_expression import Licensing
+    from license_expression import combine_expressions as le_combine_expressions
+except:
+    Licensing = None
+    le_combine_expressions = None
 
 PLAIN_URLS = (
     'https://',
@@ -102,15 +107,11 @@ def normalize_vcs_url(repo_url, vcs_tool=None):
 
     if len(repo_url.split('/')) == 2:
         # implicit github, but that's only on NPM?
-        return 'https://github.com/%(repo_url)s' % locals()
+        return f'https://github.com/{repo_url}'
 
     return repo_url
 
 
-# for legacy compat
-parse_repo_url = normalize_vcs_url
-
-
 def build_description(summary, description):
     """
     Return a description string from a summary and description
@@ -127,49 +128,90 @@ def build_description(summary, description):
     return description
 
 
-def combine_expressions(expressions, relation='AND', unique=True, licensing=Licensing()):
-    """
-    Return a combined license expression string with relation, given a list of
-    license expressions strings.
+_LICENSING = Licensing and Licensing() or None
 
-    For example::
 
-        >>> a = 'mit'
-        >>> b = 'gpl'
-        >>> combine_expressions([a, b])
-        'mit AND gpl'
-        >>> assert 'mit' == combine_expressions([a])
-        >>> combine_expressions([])
-        >>> combine_expressions(None)
-        >>> combine_expressions(('gpl', 'mit', 'apache',))
-        'gpl AND mit AND apache'
-        >>> combine_expressions(('gpl', 'mit', 'mit',))
-        'gpl AND mit'
-        >>> combine_expressions(('mit WITH foo', 'gpl', 'mit',))
-        'mit WITH foo AND gpl AND mit'
-        >>> combine_expressions(('gpl', 'mit', 'mit',), relation='OR', unique=False)
-        'gpl OR mit OR mit'
-        >>> combine_expressions(('mit', 'gpl', 'mit',))
-        'mit AND gpl'
+def combine_expressions(
+    expressions,
+    relation='AND',
+    unique=True,
+    licensing=_LICENSING,
+):
     """
-    if not expressions:
+    Return a combined license expression string with relation, given a sequence of
+    license ``expressions`` strings or LicenseExpression objects.
+    """
+    if not licensing:
+        raise Exception('combine_expressions: cannot combine combine_expressions without license_expression package.')
+    return expressions and str(le_combine_expressions(expressions, relation, unique, licensing)) or None
+
+
+def get_ancestor(levels_up, resource, codebase):
+    """
+    Return the nth-``levels_up`` ancestor Resource of ``resource`` in
+    ``codebase`` or None.
+
+    For example, with levels_up=2 and starting  with a resource path of
+    `gem-extract/metadata.gz-extract/metadata.gz-extract`,
+    then `gem-extract/` should be returned.
+    """
+    rounds = 0
+    while rounds < levels_up:
+        resource = resource.parent(codebase)
+        if not resource:
+            return
+        rounds += 1
+    return resource
+
+
+def find_root_from_paths(paths, resource, codebase):
+    """
+    Return the resource for the root directory of this filesystem or None, given
+    a ``resource`` in ``codebase`` with a list of possible resource root-
+    relative  ``paths`` (e.g. extending from the root directory we are looking
+    for).
+    """
+    for path in paths:
+        if not resource.path.endswith(path):
+            continue
+        return find_root_resource(path=path, resource=resource, codebase=codebase)
+
+
+def find_root_resource(path, resource, codebase):
+    """
+    Return the resource for the root directory of this filesystem or None, given
+    a ``resource`` in ``codebase`` with a possible resource root-relative
+    ``path`` (e.g. extending from the root directory we are looking for).
+    """
+    if not resource.path.endswith(path):
         return
+    for _seg in path.split('/'):
+        resource = resource.parent(codebase)
+        if not resource:
+            return
+    return resource
 
-    if not isinstance(expressions, (list, tuple)):
-        raise TypeError(
-            'expressions should be a list or tuple and not: {}'.format(
-                type(expressions)))
 
-    if unique:
-        # Remove duplicate element in the expressions list
-        expressions = list(dict((x, True) for x in expressions).keys())
+def yield_dependencies_from_package_data(package_data, datafile_path, package_uid):
+    """
+    Yield a Dependency for each dependency from ``package_data.dependencies``
+    """
+    from packagedcode import models
+    dependent_packages = package_data.dependencies
+    if dependent_packages:
+        yield from models.Dependency.from_dependent_packages(
+            dependent_packages=dependent_packages,
+            datafile_path=datafile_path,
+            datasource_id=package_data.datasource_id,
+            package_uid=package_uid,
+        )
 
-    if len(expressions) == 1:
-        return expressions[0]
 
-    expressions = [licensing.parse(le, simple=True) for le in expressions]
-
-    # licensing.OR or licensing.AND
-    assert relation and relation.upper() in ('AND', 'OR',)
-    relationship = getattr(licensing, relation)
-    return str(relationship(*expressions))
+def yield_dependencies_from_package_resource(resource, package_uid=None):
+    """
+    Yield a Dependency for each dependency from each package from``resource.package_data``
+    """
+    from packagedcode import models
+    for pkg_data in resource.package_data:
+        pkg_data = models.PackageData.from_dict(pkg_data)
+        yield from yield_dependencies_from_package_data(pkg_data, resource.path, package_uid)
