@@ -14,6 +14,7 @@ import time
 
 import saneyaml
 from commoncode.system import on_windows
+from packageurl import PackageURL
 
 from scancode_config import scancode_root_dir
 
@@ -40,7 +41,7 @@ def run_scan_plain(
     if not env:
         env = dict(os.environ)
 
-    scmd = u'scancode'
+    scmd = 'scancode'
     scan_cmd = os.path.join(scancode_root_dir, scmd)
     rc, stdout, stderr = execute(
         cmd_loc=scan_cmd,
@@ -93,6 +94,7 @@ def run_scan_click(
     If retry is True, wait 10 seconds after a failure and retry once
     """
     import click
+    import shutil
     from click.testing import CliRunner
     from scancode import cli
 
@@ -103,7 +105,7 @@ def run_scan_click(
 
     if monkeypatch:
         monkeypatch.setattr(click._termui_impl, 'isatty', lambda _: True)
-        monkeypatch.setattr(click , 'get_terminal_size', lambda : (80, 43,))
+        monkeypatch.setattr(shutil , 'get_terminal_size', lambda : (80, 43,))
 
     if not env:
         env = dict(os.environ)
@@ -171,6 +173,7 @@ def check_json_scan(
     regen=False,
     remove_file_date=False,
     check_headers=False,
+    remove_uuid=True,
 ):
     """
     Check the scan `result_file` JSON results against the `expected_file`
@@ -182,17 +185,27 @@ def check_json_scan(
 
     If `remove_file_date` is True, the file.date attribute is removed.
     If `check_headers` is True, the scan headers attribute is not removed.
+    If `remove_uuid` is True, removes UUID from Package and Dependency.
+    and if also `regen` is True then regenerate expected file with old UUIDs present already.
     """
-    results = load_json_result(result_file, remove_file_date)
-    if regen:
-        with open(expected_file, 'w') as reg:
-            json.dump(results, reg, indent=2, separators=(',', ': '))
-
-    expected = load_json_result(expected_file, remove_file_date)
+    results = load_json_result(location=result_file, remove_file_date=remove_file_date)
+    if remove_uuid:
+        results = remove_uuid_from_scan(results)
 
     if not check_headers:
         results.pop('headers', None)
-        expected.pop('headers', None)
+
+    if regen:
+        with open(expected_file, 'w') as reg:
+            json.dump(results, reg, indent=2, separators=(',', ': '))
+        expected = results
+    else:
+        expected = load_json_result(location=expected_file, remove_file_date=remove_file_date)
+        if remove_uuid:
+            expected = remove_uuid_from_scan(expected)
+        if not check_headers:
+            expected.pop('headers', None)
+
 
     # NOTE we redump the JSON as a YAML string for easier display of
     # the failures comparison/diff
@@ -202,6 +215,45 @@ def check_json_scan(
         assert results == expected
 
 
+def remove_uuid_from_scan(results):
+    """
+    Remove Package and Dependency UUIDs from a ``results` mapping of scan data .
+   UUID fields are generated uniquely and would cause test failures
+    when comparing results and expected.
+    """
+    for package in (results.get('packages') or []):
+        package_uid = package.get('package_uid')
+        if package_uid:
+            package['package_uid'] = purl_with_fake_uuid(package_uid)
+
+    for dependency in (results.get('dependencies') or []):
+        dependency_uid = dependency.get('dependency_uid')
+        if dependency_uid:
+            dependency['dependency_uid'] = purl_with_fake_uuid(dependency_uid)
+
+        for_package_uid = dependency.get('for_package_uid')
+        if for_package_uid:
+            dependency['for_package_uid'] = purl_with_fake_uuid(for_package_uid)
+
+    for resource in (results.get('files') or []):
+        for_packages = []
+        has_packages = False
+        for fpkg in (resource.get('for_packages') or []):
+            has_packages = True
+            for_packages.append(purl_with_fake_uuid(fpkg))
+
+        if has_packages:
+            resource['for_packages'] = for_packages
+
+    return results
+
+
+def purl_with_fake_uuid(purl):
+    purl = PackageURL.from_string(purl)
+    purl.qualifiers['uuid'] = 'fixed-uid-done-for-testing-5642512d1758'
+    return purl.to_string()
+
+
 def load_json_result(location, remove_file_date=False):
     """
     Load the JSON scan results file at `location` location as UTF-8 JSON.
